% ==========================
% =        Hyperion        =
% ==========================
%
% Hyperion 0.1
% Graph parser for hyperedge replacement grammars
%
% Copyright (C) 2008 Richard Moot (Richard.Moot@labri.fr)
% Copyright (C) 2008 CNRS         (http://www.cnrs.fr)
% Copyright (C) 2008 INRIA        (http://www.inria.fr

% This library is free software; you can redistribute it and/or
% modify it under the terms of the GNU Lesser General Public
% License as published by the Free Software Foundation; either
% version 2.1 of the License, or (at your option) any later version.

% This library is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% Lesser General Public License for more details.

% You should have received a copy of the GNU Lesser General Public
% License along with this library; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

:- import select/3, append/3, member/2, length/2 from basics.
:- import numbervars/3 from num_vars.
:- import format/2, format/3 from format.
:- import variant/2 from subsumes.

:- table parse/4.

% ==========================
% =         parser         =
% ==========================

parse_examples :-
	findall(N-G, example(N, G), L),
	start(S),
	format('Start symbol: ~w~n', [S]),
	parse_examples(L, S).

parse_examples([], _) :-
	format('~nDone!~n', []).
parse_examples([N-G|Es], S) :-
	findall(T, parse(S, G, [], T), L),
	length(L, Len),
	format('Example graph ~w: ', [N]),
	print_solution_string(Len),
	parse_examples(Es, S).

print_solution_string(Sol) :-
   (
       Sol =:= 0
   ->
	format('* No solutions! *~n', [])
    ;
	Sol =:= 1
    ->
	format('1 solution.~n', [])
    ;
	format('~w solutions.~n', [Sol])
    ).


parse_example(N) :-
	parse_example(N, T),
	print_tree(T).

parse_example(N, t([],[S-T])) :-
	example(N, G),
	start(S),
	parse(S, G, [], T).

% This is the basic top-down parser for hyperedge replacement grammars
% described in Lauteman 1990. Remember that it requires your grammar
% to have no epsilon-rules.

parse(NT, G0, G, Tr) :-
	match_rule(NT, G0, G, Tr).

match_rule(NT, G0, G, t(Ts, NTrs)) :-
	rule(NT, NTs, Ts),
	select_all(Ts, G0, G1),
	parse_all(NTs, G1, G, NTrs).

parse_all([], G, G, []).
parse_all([N|Ns], G0, G, [N-Tr|NTrs]) :-
	parse(N, G0, G1, Tr),
	parse_all(Ns, G1, G, NTrs).

select_all([], G, G).
select_all([X|Xs], Ys0, Ys) :-
	select(X, Ys0, Ys1),
	select_all(Xs, Ys1, Ys).

% = sketches, needs to be completely redone

bottom_up_parse_example(N) :-
	bottom_up_parse_example(N, T),
	print_tree(T).

bottom_up_parse_example(N, T) :-
	example(N, G),
	bottom_up_parse(G, T).

initialize_chart(G0, _T) :-
	rule(NT, [], Ts),
	select_all(Ts, G0, G),
	bottom_up_parse(NT, Ts, G).

bottom_up_parse(NT, G0, G) :-
    (
	start(_S)
    ->
	G = G0
    ;
	rule(NT, NTs, Ts),
	select_all(Ts, G0, G1),
	select_all(NTs, G1, G),
	parse_rest(G, [], _T)
    ).

% ======================
% =     Text output    =
% ======================

pp_tree(T) :-
	print_tree(T, 0).
pp_tree(t(T,Ts), Tab) :-
	tab(Tab),
	format('T: ~w~n', [T]),
	print_trees(Ts, Tab).

pp_trees([], _).
pp_trees([N-Tr|NTrs],Tab0) :-
	tab(Tab0), 
	format('N: ~w~n', [N]),
	Tab is Tab0 + 4,
	tab(Tab), 
	format('{~n',[]),
	print_tree(Tr, Tab),
	tab(Tab),
	format('}~n',[]),
	print_trees(NTrs, Tab0).


% ==========================
% = grammar transformation =
% ==========================

% = file_to_grammar(+FileName, -Grammar)
%
% True if Grammar is a Prolog term representing the hyperedge replacement
% grammar in File. Prints out a warning message in the case of unknown
% predicates.

file_to_grammar(File, Grammar) :-
	open(File, read, Stream),
	read_grammar_file(Stream,List),
	close(Stream),
	list_to_grammar(List, Grammar).

% = grammar_to_file(+FileName, +Grammar)
%
% Outputs a Prolog term representing a hyperedge replacement grammar
% to FileName.

grammar_to_file(File, Grammar) :-
	open(File, write, Stream),
	output_grammar_file(Stream, Grammar),
	close(Stream).

% = cleanup_grammar(+InFile,+OutFile)
%
% Performs some minor corrections on the InFile and outputs the
% resulting grammar to OutFile.
%
% Currently, the following corrections are performed.
% - terminals and nonterminals which are used in the grammar but not
%   declared as such (by terminal/1 and nonterminal/1) will have the
%   proper declarations added.
% - some basic hyperedge_labels/1 declarations for terminals and
%   nonterminals which don't have one will be added. This prevents
%   the dot output from failing.
% - multiple start symbols will be replaced by a single fresh symbol
%   and rewrite rules to all of the old start symbols will be added.

cleanup_grammar(InFile, OutFile) :-
	file_to_grammar(InFile, Grammar0),
	cleanup(Grammar0, Grammar),
	grammar_to_file(OutFile, Grammar).

% = remove_epsilon(+InFile,-OutFile)
%
% Since the parser requires grammars to be epsilon-free, this
% predicate compiles out all epsilon rules from the grammar in
% InFile and outputs the resulting grammar in OutFile.
% Needs to be more thoroughly tested and made more efficient.

remove_epsilon(InFile, OutFile) :-
	file_to_grammar(InFile, Grammar0),
	remove_epsilon1(Grammar0, Grammar),
	grammar_to_file(OutFile, Grammar).

remove_epsilon1(Grammar0, g(S,N,T,H,R,E)) :-
	cleanup(Grammar0, g(S,N,T,H,R0,E)),
	find_epsilon(R0, Epsilon, R1),
	remove_epsilon1(Epsilon, [], [], R1, R).

remove_epsilon1([], New, Done, R0, R) :-
	remove_epsilon2(New, Done, R0, R).
remove_epsilon1([E|Es], N0, D0, R0, R) :-
	add_new_rules(E, N0, N, D0, D, R0, R1),
	remove_epsilon1(Es, N, D, R1, R).

remove_epsilon2([], _, R, R).
remove_epsilon2([E|Es], D0, R0, R) :-
	add_new_rules(E, [], N, D0, D, R0, R1),
	remove_epsilon1(Es, N, D, R1, R).

add_new_rules(E, N0, N, D0, D, R0, R) :-
    (
	member(E0, D0),
	variant(E0, E)
    ->
	D = D0,
	N = N0,
	R = R0
    ;
	D = [E|D0],
	match_rules(R0, E, D, N0, N, [], R)
    ).

match_rules([], _, _, N, N, AR, AR).
match_rules([rule(A,B0,C)|R0], E, D, N0, N, AR0, AR) :-
 	findall(rule(A,B,C), select(E,B0,B), L),
	copy_term(L, Copy),
	numbervars(Copy, 0, _),
	output_list(Copy),
	add_rules(L, D, N0, N1, AR0, R1),
	append(R1, R0, R),
	match_rules(R, E, D, N1, N, [rule(A,B0,C)|AR0], AR).

add_rules([], _, N, N, _, []).
add_rules([Rule|Rules], D, N0, N, AR, R0) :-
	Rule = rule(A,B,C),
    (
	member(Rule0, AR),
	variant(Rule0, Rule)
    ->
	R = R0
    ;
	R0 = [Rule|R]
    ),
    (
	B = [],
	C = []
    ->
       (
	   member(A0, D),
	   variant(A0, A)
       ->
	   N1 = N0
       ;
	   N1 = [A|N0]
       )
    ;
	N1 = N0
    ),
	add_rules(Rules, D, N1, N, AR, R).


list_to_grammar(L0, g(S, N, T, H, R, E)) :-
	select_instances(start(_), S, L0, L1),
	select_instances(nonterminal(_), N, L1, L2),
	select_instances(terminal(_), T, L2, L3),
	select_instances(hyperedge_labels(_), H, L3, L4),
	select_instances(rule(_,_,_), R, L4, L5),
	select_instances(example(_,_), E, L5, L),
	warn_unused(L).

warn_unused([]).
warn_unused([U|Us]) :-
	format('{Warning: unused grammar clause: ~w}~n', [U]),
	warn_unused(Us).

find_epsilon([], [], []).
find_epsilon([rule(A,B,C)|Rs0], Es0, Rs1) :-
    (
	B = [],
	C = []
    ->
	Es0 = [A|Es],
	Rs1 = Rs
    ;
	Es = Es0,
	Rs1 = [rule(A,B,C)|Rs]
    ),
	find_epsilon(Rs0, Es, Rs).

cleanup(g(S0, N0, T0, H0, R0, E), g(S, N, T, H, R, E)) :-
	rule_coherence(R0, N0, T0, R1, N1, T),
	remove_multiple_starts(S0, S, R1, R, N1, N),
	add_hyperedge_labels(N, T, H0, H).

% = add_hyperedge_labels(+NonTerminals,+Terminals,+HELabels,-HELabels)
%
% hyperedge_labels declarations are necessary for portraying a graph,
% this predicate adds some very basic declarations to make sure
% graph printing won't fail even if a user has forgotten these
% declarations.

add_hyperedge_labels([], T, H0, H) :-
	add_hyperedge_labels(T, H0, H).
add_hyperedge_labels([nonterminal(N0)|Ns], T, H0, H) :-
	functor(N0, F, A),
	functor(N, F, A),
    (
	member(hyperedge_labels(N), H0)
    ->
	H1 = H0
    ;
	create_labels(1, A, t, N),
	H1 = [hyperedge_labels(N)|H0]
    ),
	add_hyperedge_labels(Ns, T, H1, H).

add_hyperedge_labels([], H, H).
add_hyperedge_labels([terminal(T0)|Ts], H0, H) :-
	functor(T0, F, A),
	functor(T, F, A),
    (
	member(hyperedge_labels(T), H0)
    ->
	H1 = H0
    ;
	create_labels(1, A, t, T),
	H1 = [hyperedge_labels(T)|H0]
    ),
	add_hyperedge_labels(Ts, H1, H).

create_labels(A0, A, L, T) :-
    (
	A0 > A
    ->
	true
    ;
	arg(A0, T, L-A0),
	A1 is A0+1,
	create_labels(A1, A, f, T)
    ).
	
% = remove_multiple_starts(+Starts,-Start,+Rules,-Rules,+NTs,-NTs)
%
% Hyperion requires a grammar to have a single start symbol. However
% it is sometimes convenient to specify multiple start symbols in a
% grammar. In case multiple start symbols are found in the grammar,
% a new start symbol is introduced (sN for the smallest N not already
% in the set of nonterminals) and a rewrite rule from this start
% symbol to another start symbol.
% If no start symbol is found, an error message is produced.

remove_multiple_starts(S0, S, R0, R, N0, N) :-
    (
	S0 = [_,_|_]
    ->
	format('{Warning: multiple start nonterminals found, transforming grammar}~n', []),
	remove_multiple_starts1(S0, S, R0, R, N0, N)
    ;
	S0 = []
    ->
	format('{Error: no start nonterminals found}~n', []),
	fail
    ;
	S0 = [S],
	R = R0,
	N = N0
    ).

remove_multiple_starts1(S0, S, R0, R, N0, N) :-
	new_start_symbol(N0, N, S, 0),
	add_rules(S0, S, R0, R).

add_rules([], _, R, R).
add_rules([N|Ns], S, R0, R) :-
	R1 = [rule(S, [N], [])|R0],
	add_rules(Ns, S, R1, R).

new_start_symbol(N0, N, S, Num0) :-
	name(Num0, L),
	name(Start, [115|L]),
    (
	member(Start, N0)
    ->
	Num is Num0 + 1,
	new_start_symbol(N0, N, S, Num)
    ;
	S = Start,
	N = [S|N0]
    ).

% = rule_coherence(+Rules,+NTs,+Ts,-Rules,-NTs,-Ts)

rule_coherence([], N, T, [], N, T).
rule_coherence([rule(A,B,C)|Rs0], N0, T0, [rule(A,B,C)|Rs], N, T) :-
	add_nonterminal(A, N0, N1),
	nonterminal_list(B, N1, N2),
	terminal_list(C, T0, T1),
	rule_coherence(Rs0, N2, T1, Rs, N, T).

nonterminal_list([], N, N).
nonterminal_list([NT|NTs], N0, N) :-
	add_nonterminal(NT, N0, N1),
	nonterminal_list(NTs, N1, N).

terminal_list([], T, T).
terminal_list([Trm|Trms], T0, T) :-
	add_terminal(Trm, T0, T1),
	terminal_list(Trms, T1, T).

add_nonterminal(NT, N0, N) :-
	functor(NT, F, A),
	functor(NT0, F, A),
    (
	member(nonterminal(NT0), N0)
    ->
	N = N0
    ;
	format('{Warning: missing nonterminal declaration for ~w/~w has been added}~n', [F,A]),
	N = [nonterminal(NT0)|N0]
    ).

add_terminal(Trm, T0, T) :-
	functor(Trm, F, A),
	functor(Trm0, F, A),
    (
	member(terminal(Trm0), T0)
    ->
	T = T0
    ;
	format('{Warning: missing terminal declaration for ~w/~w has been added}~n', [F,A]),
	T = [terminal(Trm0)|T0]
    ).

select_instances(Term, Terms, L0, L) :-
	functor(Term, F, A),
	functor(Term0, F, A),
    (
	select(Term0, L0, L1)
    ->
	Terms = [Term0|Terms0],
	select_instances(Term, Terms0, L1, L)
    ;
	Terms = [],
	L = L0
    ).

read_grammar_file(Stream, Gr) :-
	read_term(Stream, Term, [variables(_Vars),variable_names(_VN)]),
	read_grammar_file(Term, Stream, Gr).

read_grammar_file(end_of_file, _, []) :-
	!.
read_grammar_file(Term, Stream, [Term|Gr]) :-
	read_grammar_file(Stream, Gr).

output_grammar_file(Stream, g(S,N,T,H,R,E)) :-
	format(Stream, '% ====================~n% =   start symbol   =~n% ====================~2n', []),
        output_list([S], Stream),
	format(Stream, '% ====================~n% =   nonterminals   =~n% ====================~2n', []),
        output_list(N, Stream),
	format(Stream, '% ====================~n% =    terminals     =~n% ====================~2n', []),
        output_list(T, Stream),
	format(Stream, '% ====================~n% = hyperedge labels =~n% ====================~2n', []),
        output_list(H, Stream),
	format(Stream, '% ====================~n% =       rules      =~n% ====================~2n', []),
        output_list(R, Stream),
	format(Stream, '% ====================~n% =  example graphs  =~n% ====================~2n', []),
        output_list(E, Stream).


output_list([]) :-
	format('~n', []).
output_list([X|Xs]) :-
	format('~w.~n', [X]),
	output_list(Xs).

output_list([], Stream) :-
	format(Stream, '~n', []).
output_list([X|Xs], Stream) :-
	numbervars(X, 0, _),
	format(Stream, '~w.~n', [X]),
	output_list(Xs, Stream).

% ==========================
% =    Grail interface     =
% ==========================

% sequent_to_hypergraph(+ListOfFormulas, +Conclusion, -HyperGraph)
%
% Transforms a sequent in Grail notation into a hypergraph in Hyperion
% notation. The current translation supports two binary modes `0' and
% `1' but no unary connectives yet. This translation function is
% intended to be used together with the grammar nldr.P, which provides
% several examples.

sequent_to_hypergraph(Fs, C, L) :-
	sequent_to_hypergraph(Fs, C, L, []).

sequent_to_hypergraph(Fs, C) -->
	sequent_to_hypergraph(Fs, C, 0, _).

sequent_to_hypergraph([], C, N0, N) -->
	[c(N0)],
	formula_to_hypergraph_pos(C, N0, N).
sequent_to_hypergraph([F|Fs], C, N0, N) -->
	[h(N0)],
	formula_to_hypergraph_neg(F, N0, N1),
	sequent_to_hypergraph(Fs, C, N1, N).

formula_to_hypergraph_pos(np, N0, N) -->
	{ N is N0+1 },
	[ax_np_p(N0)].
formula_to_hypergraph_pos(vp, N0, N) -->
	{ N is N0+1 },
	[ax_vp_p(N0)].
formula_to_hypergraph_pos(vp1, N0, N) -->
	{ N is N0+1 },
	[ax_vp1_p(N0)].
formula_to_hypergraph_pos(n, N0, N) -->
	{ N is N0+1 },
	[ax_n_p(N0)].
formula_to_hypergraph_pos(s, N0, N) -->
	{ N is N0+1 },
	[ax_s_p(N0)].
formula_to_hypergraph_pos(lit(A), N0, N) -->
	formula_to_hypergraph_pos(A, N0, N).
formula_to_hypergraph_pos(p(I,A,B), N0, N) -->
	formula_to_hypergraph_pos_p(I, A, B, N0, N).
formula_to_hypergraph_pos(dl(I,A,B), N0, N) -->
	formula_to_hypergraph_pos_dl(I, A, B, N0, N).
formula_to_hypergraph_pos(dr(I,A,B), N0, N) -->
	formula_to_hypergraph_pos_dr(I, A, B, N0, N).

formula_to_hypergraph_pos_p(0, A, B, N0, N) -->
	{ N1 is N0+1 },
	formula_to_hypergraph_pos(A, N1, N2),
	formula_to_hypergraph_pos(B, N2, N),
	[circ0(N1,N2,N0)].
formula_to_hypergraph_pos_p(1, A, B, N0, N) -->
	{ N1 is N0+1 },
	formula_to_hypergraph_pos(A, N1, N2),
	formula_to_hypergraph_pos(B, N2, N),
	[circ1(N1,N2,N0)].
formula_to_hypergraph_pos_dl(0, A, B, N0, N) -->
	{ N1 is N0+1 },
	formula_to_hypergraph_neg(A, N1, N2),
	formula_to_hypergraph_pos(B, N2, N),
	[rdl0(N1,N0,N2)].
formula_to_hypergraph_pos_dl(1, A, B, N0, N) -->
	{ N1 is N0+1 },
	formula_to_hypergraph_neg(A, N1, N2),
	formula_to_hypergraph_pos(B, N2, N),
	[rdl1(N1,N0,N2)].
formula_to_hypergraph_pos_dr(0, A, B, N0, N) -->
	{ N1 is N0+1 },
	formula_to_hypergraph_pos(A, N1, N2),
	formula_to_hypergraph_neg(B, N2, N),
	[rdr0(N0,N2,N1)].
formula_to_hypergraph_pos_dr(1, A, B, N0, N) -->
	{ N1 is N0+1 },
	formula_to_hypergraph_pos(A, N1, N2),
	formula_to_hypergraph_neg(B, N2, N),
	[rdr1(N0,N2,N1)].

formula_to_hypergraph_neg(np, N0, N) -->
	{ N is N0+1 },
	[ax_np_n(N0)].
formula_to_hypergraph_neg(vp, N0, N) -->
	{ N is N0+1 },
	[ax_vp_n(N0)].
formula_to_hypergraph_neg(vp1, N0, N) -->
	{ N is N0+1 },
	[ax_vp1_n(N0)].
formula_to_hypergraph_neg(n, N0, N) -->
	{ N is N0+1 },
	[ax_n_n(N0)].
formula_to_hypergraph_neg(s, N0, N) -->
	{ N is N0+1 },
	[ax_s_n(N0)].
formula_to_hypergraph_neg(lit(A), N0, N) -->
	formula_to_hypergraph_neg(A, N0, N).
formula_to_hypergraph_neg(p(I,A,B), N0, N) -->
	formula_to_hypergraph_neg_p(I, A, B, N0, N).
formula_to_hypergraph_neg(dl(I,A,B), N0, N) -->
	formula_to_hypergraph_neg_dl(I, A, B, N0, N).
formula_to_hypergraph_neg(dr(I,A,B), N0, N) -->
	formula_to_hypergraph_neg_dr(I, A, B, N0, N).

formula_to_hypergraph_neg_p(0, A, B, N0, N) -->
	{ N1 is N0+1 },
	formula_to_hypergraph_neg(A, N1, N2),
	formula_to_hypergraph_neg(B, N2, N),
	[lprod0(N1,N2,N0)].
formula_to_hypergraph_neg_p(1, A, B, N0, N) -->
	{ N1 is N0+1 },
	formula_to_hypergraph_neg(A, N1, N2),
	formula_to_hypergraph_neg(B, N2, N),
	[lprod1(N1,N2,N0)].
formula_to_hypergraph_neg_dl(0, A, B, N0, N) -->
	{ N1 is N0+1 },
	formula_to_hypergraph_pos(A, N1, N2),
	formula_to_hypergraph_neg(B, N2, N),
	[circ0(N1,N0,N2)].
formula_to_hypergraph_neg_dl(1, A, B, N0, N) -->
	{ N1 is N0+1 },
	formula_to_hypergraph_pos(A, N1, N2),
	formula_to_hypergraph_neg(B, N2, N),
	[circ1(N1,N0,N2)].
%  N0       N2
% A/B       B
%    \     /
%     \   /
%      [o]
%       |
%       |
%       A
%       N1

formula_to_hypergraph_neg_dr(0, A, B, N0, N) -->
	{ N1 is N0+1 },
	formula_to_hypergraph_neg(A, N1, N2),
	formula_to_hypergraph_pos(B, N2, N),
	[circ0(N0,N2,N1)].
formula_to_hypergraph_neg_dr(1, A, B, N0, N) -->
	{ N1 is N0+1 },
	formula_to_hypergraph_neg(A, N1, N2),
	formula_to_hypergraph_pos(B, N2, N),
	[circ1(N0,N2,N1)].

% lg_sequent_to_hypergraph(+ListOfFormulas, +Conclusion, -HyperGraph)
%
% transforms an LG sequent into a hypergraph.

lg_sequent_to_hypergraph(Fs, C, L) :-
	lg_sequent_to_hypergraph(Fs, C, L, []).

lg_sequent_to_hypergraph(Fs, C) -->
	lg_sequent_to_hypergraph(Fs, C, 0, _).

lg_sequent_to_hypergraph([], C, N0, N) -->
	[c(N0)],
	lg_formula_to_hypergraph_pos(C, N0, N).
lg_sequent_to_hypergraph([F|Fs], C, N0, N) -->
	[h(N0)],
	lg_formula_to_hypergraph_neg(F, N0, N1),
	lg_sequent_to_hypergraph(Fs, C, N1, N).

lg_formula_to_hypergraph_pos(np, N0, N) -->
	{ N is N0+1 },
	[ax_np_p(N0)].
lg_formula_to_hypergraph_pos(vp, N0, N) -->
	{ N is N0+1 },
	[ax_vp_p(N0)].
lg_formula_to_hypergraph_pos(vp1, N0, N) -->
	{ N is N0+1 },
	[ax_vp1_p(N0)].
lg_formula_to_hypergraph_pos(n, N0, N) -->
	{ N is N0+1 },
	[ax_n_p(N0)].
lg_formula_to_hypergraph_pos(s, N0, N) -->
	{ N is N0+1 },
	[ax_s_p(N0)].
lg_formula_to_hypergraph_pos(lit(A), N0, N) -->
	lg_formula_to_hypergraph_pos(A, N0, N).
lg_formula_to_hypergraph_pos(p(A,B), N0, N) -->
	{ N1 is N0+1 },
	lg_formula_to_hypergraph_pos(A, N1, N2),
	lg_formula_to_hypergraph_pos(B, N2, N),
	[circ(N1,N2,N0)].
lg_formula_to_hypergraph_pos(dl(A,B), N0, N) -->
	{ N1 is N0+1 },
	lg_formula_to_hypergraph_neg(A, N1, N2),
	lg_formula_to_hypergraph_pos(B, N2, N),
	[rdl(N1,N0,N2)].
lg_formula_to_hypergraph_pos(dr(A,B), N0, N) -->
	{ N1 is N0+1 },
	lg_formula_to_hypergraph_pos(A, N1, N2),
	lg_formula_to_hypergraph_neg(B, N2, N),
	[rdr(N0,N2,N1)].
lg_formula_to_hypergraph_pos(g(A,B), N0, N) -->
	{ N1 is N0+1 },
	lg_formula_to_hypergraph_pos(A, N1, N2),
	lg_formula_to_hypergraph_pos(B, N2, N),
	[rplus(N1,N2,N0)].
lg_formula_to_hypergraph_pos(gl(A,B), N0, N) -->
	{ N1 is N0+1 },
	lg_formula_to_hypergraph_neg(A, N1, N2),
	lg_formula_to_hypergraph_pos(B, N2, N),
	[semi(N1,N0,N2)].
lg_formula_to_hypergraph_pos(gr(A,B), N0, N) -->
	{ N1 is N0+1 },
	lg_formula_to_hypergraph_pos(A, N1, N2),
	lg_formula_to_hypergraph_neg(B, N2, N),
	[semi(N0,N2,N1)].

lg_formula_to_hypergraph_neg(np, N0, N) -->
	{ N is N0+1 },
	[ax_np_n(N0)].
lg_formula_to_hypergraph_neg(vp, N0, N) -->
	{ N is N0+1 },
	[ax_vp_n(N0)].
lg_formula_to_hypergraph_neg(vp1, N0, N) -->
	{ N is N0+1 },
	[ax_vp1_n(N0)].
lg_formula_to_hypergraph_neg(n, N0, N) -->
	{ N is N0+1 },
	[ax_n_n(N0)].
lg_formula_to_hypergraph_neg(s, N0, N) -->
	{ N is N0+1 },
	[ax_s_n(N0)].
lg_formula_to_hypergraph_neg(lit(A), N0, N) -->
	lg_formula_to_hypergraph_neg(A, N0, N).
lg_formula_to_hypergraph_neg(p(A,B), N0, N) -->
	{ N1 is N0+1 },
	lg_formula_to_hypergraph_neg(A, N1, N2),
	lg_formula_to_hypergraph_neg(B, N2, N),
	[lprod(N1,N2,N0)].
lg_formula_to_hypergraph_neg(dl(A,B), N0, N) -->
	{ N1 is N0+1 },
	lg_formula_to_hypergraph_pos(A, N1, N2),
	lg_formula_to_hypergraph_neg(B, N2, N),
	[circ(N1,N0,N2)].
lg_formula_to_hypergraph_neg(dr(A,B), N0, N) -->
	{ N1 is N0+1 },
	lg_formula_to_hypergraph_neg(A, N1, N2),
	lg_formula_to_hypergraph_pos(B, N2, N),
	[circ(N0,N2,N1)].
lg_formula_to_hypergraph_neg(g(A,B), N0, N) -->
	{ N1 is N0+1 },
	lg_formula_to_hypergraph_neg(A, N1, N2),
	lg_formula_to_hypergraph_neg(B, N2, N),
	[semi(N1,N2,N0)].
lg_formula_to_hypergraph_neg(gl(A,B), N0, N) -->
	{ N1 is N0+1 },
	lg_formula_to_hypergraph_pos(A, N1, N2),
	lg_formula_to_hypergraph_neg(B, N2, N),
	[lgl(N1,N0,N2)].
lg_formula_to_hypergraph_neg(gr(A,B), N0, N) -->
	{ N1 is N0+1 },
	lg_formula_to_hypergraph_neg(A, N1, N2),
	lg_formula_to_hypergraph_pos(B, N2, N),
	[lgr(N0,N2,N1)].

% ======================
% = GraphViz interface =
% ======================

print_tree(T) :-
	format('digraph deriv {~2n  compound=true;~n', []),
	fresh_node([T], M),
	print_tree(T, M, _),
	format('~n}~n', []).

print_tree(T0, M0, M) :-
%	renumber_tree(T0, T, M0, M1, _L, []),
	T = T0,
	M1 = M0,
	print_tree1(T, M1, M).

print_tree1(t(T,NTs), M0, M) :-
	M1 is M0 + 1,
	format('~n  subgraph cluster~w {~2n', [M0]),
	print_terminals(T, M1, M2),
	print_rule_nts(NTs, M2, M3),
	format('    color=blue;~2n  }~n', []),
	print_trees(NTs, M3, M).

print_rule_nts([], M, M).
print_rule_nts([NT-_|NTs], M0, M) :-
	print_hyperedge(NT, M0, M1),
	print_rule_nts(NTs, M1, M).

print_trees([], M, M).
print_trees([NT-T|NTs], M0, M) :-
	M1 is M0 + 1,
%	format('~n  subgraph cluster~w {~2n', [M0]),
	print_tree(T, M1, M2),
	format('~n', []),
	external_nodes(NT,Ext),
	print_external(Ext),
%	format('    color=lightgrey;~2n  }~n', []),
	print_trees(NTs, M2, M).
	
renumber_tree(t(T0,NTs0), t(T,NTs), M0, M, L0, L) :-
	renumber_terminals(T0, T, M0, M1, L0, L1),
	renumber_nonterminals(NTs0, NTs, M1, M, L1, L).

renumber_terminals([], [], M, M, L, L).
renumber_terminals([T|Ts], [U|Us], M0, M, L0, L) :-
	renumber_term(T, U, M0, M1, L0, L1),
	renumber_terminals(Ts, Us, M1, M, L1, L).

renumber_nonterminals([], [], M, M, L, L).
renumber_nonterminals([N0-T|NTs0], [N-T|NTs], M0, M, L0, L) :-
	renumber_term(N0, N, M0, M1, L0, L1),
	renumber_nonterminals(NTs0, NTs, M1, M, L1, L).

renumber_term(T0, T, M0, M, L0, L) :-
	T0 =.. [F|A0],
	renumber_term1(A0, A, M0, M, L0, L),
	T =.. [F|A].

renumber_term1([], [], M, M, L, L).
renumber_term1([A|As], [M0|Bs], M0, M, [M0-A|L0], L) :-
	M1 is M0 + 1,
	renumber_term1(As, Bs, M1, M, L0, L).

% = print_hypergraph(+Graph)

print_hypergraph(G) :-
	format('digraph hyper {~n', []),
	print_hypergraph1(G),
	format('~n}~n', []).

% = print_hypergraph(+Graph,+External)

print_hypergraph(G, Ext) :-
	format('digraph hyper {~n', []),
	print_hypergraph1(G),
	print_external(Ext),
	format('~n}~n', []).

print_hypergraph1(G) :-
	fresh_node(G, N),
	print_hypergraph2(G, N).

print_hypergraph2([], _).
print_hypergraph2([E|Es], N0) :-
	print_hyperedge(E, N0, N),
	print_hypergraph2(Es, N).

% = fresh_node(+HyperEdges,-Min,+Max+1)
%
% searches the hypergraph for all integers used as vertex numbers
% and returns the maximum integer used plus one.

fresh_node(Es, N) :-
	fresh_node(Es, 0, N).

fresh_node([], N0, N) :-
	N is N0 + 1.
fresh_node([E|Es], N0, N) :-
	E =.. [_|As],
	fresh_node1(As, N0, N1),
	fresh_node(Es, N1, N).

fresh_node1([], N, N).
fresh_node1([A|As], N0, N) :-
    (
	integer(A),
	A > N0
    ->
	N1 = A
    ;
	compound(A)
    ->
	A =.. [_|Bs],
	fresh_node1(Bs, N0, N1)
    ;
	N1 = N0
    ),
	fresh_node1(As, N1, N).

% = print_hyperedge(+Hyperedge,+FreshNodeIn,-FreshNodeOut)

print_hyperedge(N, M0, M) :-
	N =.. [F|As],
	functor(N, F, Ar),
	functor(L, F, Ar),
	hyperedge_labels(L),
	L =.. [F|Bs],
	M is M0 + 1,
	format('    ~w [shape=rectangle,label="~w"];~n', [M0,F]),
	print_vertex_list(As, Bs, M0).

% = print_rules
%
% outputs all rules in the grammar file in dot format.

print_rules :-
	findall(rule(NT,NTs,Ts), rule(NT, NTs, Ts), Ls),
	format('digraph rules {~2n  compound=true;~n', []),
	print_rules(Ls, 0, 0),
	format('~n}~n', []).

print_rules([], _, _).
print_rules([R|Rs], N0, M0) :-
	print_rule(R, N0, N, M0, M),
	print_rules(Rs, N, M).

print_rule(rule(NT,NTs,Ts), N0, N, M0, M) :-
	numbervars(NT, N0, N1),
	numbervars(NTs, N1, N2),
	numbervars(Ts, N2, N),
	M1 is M0 + 1,
	format('~n  subgraph cluster~w {~2n', [M0]),
	print_lhs(NT, M1, M2),
	format('    color=blue;~2n  }~n', []),
	M3 is M2 + 1,
	format('~n  subgraph cluster~w {~2n', [M2]),
	print_rhs(NTs, Ts, M3, M),
	format('~n', []),
	external_nodes(NT,Ext),
	print_external(Ext),
	format('    color=lightgrey;~2n  }~2n  ~w -> ~w [ltail=cluster~w,lhead=cluster~w];~n', [M1,M3,M0,M2]).

external_nodes(NT, Ext) :-
	NT =.. [_|Ext].

print_external([]) :-
	format('~n', []).
print_external([V|Vs]) :-
	format('    ~w [shape=doublecircle,label="~w"];~n', [V,V]),
	print_external(Vs).

print_lhs(NT, M0, M) :-
	NT =.. [F|As],
	functor(NT, F, Ar),
	functor(L, F, Ar),
	hyperedge_labels(L),
	L =.. [F|Bs],
	M1 is M0+1,
	format('    ~w [shape=rectangle,label="~w"];~n', [M0,F]),
	print_vertex_list(As, Bs, M0, M1, M).

print_vertex_list([], [], _) :-
	format('~n', []).
print_vertex_list([V|Vs], [O-L|Ls], HE) :-
	format('    ~w [shape=circle,label="~w"];~n', [V,V]),
	orientation(O, HE, V, In, Out, HT),
	format('    ~w -> ~w [arrowhead=none,~w="~w"];~n', [In,Out,HT,L]),
	print_vertex_list(Vs, Ls, HE).


print_vertex_list([], [], _, M, M) :-
	format('~n', []).
print_vertex_list([V|Vs], [O-L|Ls], HE, M0, M) :-
	M1 is M0+1,
	format('    ~w [shape=circle,label="~w"];~n', [M0,V]),
	orientation(O, HE, M0, In, Out, HT),
	format('    ~w -> ~w [arrowhead=none,~w="~w"];~n', [In,Out,HT,L]),
	print_vertex_list(Vs, Ls, HE, M1, M).

orientation(f, A, B, A, B, taillabel).
orientation(t, A, B, B, A, headlabel).

print_rhs(NTs, Ts, M0, M) :-
	print_nonterminals(NTs, M0, M1),
	print_terminals(Ts, M1, M).

print_nonterminals([], M, M).
print_nonterminals([N|Ns], M0, M) :-
	print_hyperedge(N, M0, M1),
	print_nonterminals(Ns, M1, M).

print_terminals([], M, M).
print_terminals([T|Ts], M0, M) :-
	print_hyperedge(T, M0, M1),
	print_terminals(Ts, M1, M).

test :-
	spy(add_hyperedge_labels),
	remove_epsilon('flowchart.P', 'flowchart_without_e.P').