
% ====================
% =        LG        =
% ====================

% Hyperedge replacement grammar for the Lambek-Grishin calculus
% with the class IV interaction postulates. This grammar
% illustrates the embedding of Tree Adjoining Grammars described
% in my paper "Lambek Grammars, Hyperedge Replacement Grammars
% and Tree Adjoining Grammars".
%
% Copyright (C) 2008 Richard Moot (Richard.Moot@labri.fr)
% Copyright (C) 2008 CNRS         (http://www.cnrs.fr)
% Copyright (C) 2008 INRIA        (http://www.inria.fr

% This library is free software; you can redistribute it and/or
% modify it under the terms of the GNU Lesser General Public
% License as published by the Free Software Foundation; either
% version 2.1 of the License, or (at your option) any later version.

% This library is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% Lesser General Public License for more details.

% You should have received a copy of the GNU Lesser General Public
% License along with this library; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

% ====================
% =   start symbol   =
% ====================

start(s).

% ====================
% =   nonterminals   =
% ====================

nonterminal(s).
nonterminal(t(_,_)).
nonterminal(v(_,_)).

% ====================
% =     terminals    =
% ====================

terminal(h(_)).
terminal(c(_)).

terminal(ax_np_n(_)).
terminal(ax_np_p(_)).
terminal(ax_vp_n(_)).
terminal(ax_vp_p(_)).
terminal(ax_vp1_n(_)).
terminal(ax_vp1_p(_)).
terminal(ax_n_n(_)).
terminal(ax_n_p(_)).
terminal(ax_s_n(_)).
terminal(ax_s_p(_)).

terminal(circ(_,_,_)).
terminal(lprod(_,_,_)).
terminal(rdr(_,_,_)).
terminal(rdl(_,_,_)).

terminal(semi(_,_,_)).
terminal(rplus(_,_,_)).
terminal(lgr(_,_,_)).
terminal(lgl(_,_,_)).

% ====================
% = hyperedge labels =
% ====================

hyperedge_labels(s).

hyperedge_labels(t(t-1,f-0)).
hyperedge_labels(v(t-1,f-0)).

hyperedge_labels(h(f-0)).
hyperedge_labels(c(t-0)).

hyperedge_labels(circ(t-1,t-2,f-0)).
hyperedge_labels(lprod(f-1,f-2,t-0)).
hyperedge_labels(rdr(f-1,f-2,t-0)).
hyperedge_labels(rdl(f-1,f-2,t-0)).

hyperedge_labels(semi(f-1,f-2,t-0)).
hyperedge_labels(rplus(t-1,t-2,f-0)).
hyperedge_labels(lgr(t-1,t-2,f-0)).
hyperedge_labels(lgl(t-1,t-2,f-0)).

hyperedge_labels(ax_np_n(t-0)).
hyperedge_labels(ax_np_p(f-0)).
hyperedge_labels(ax_pp_n(t-0)).
hyperedge_labels(ax_pp_p(f-0)).
hyperedge_labels(ax_vp_n(t-0)).
hyperedge_labels(ax_vp_p(f-0)).
hyperedge_labels(ax_vp1_n(t-0)).
hyperedge_labels(ax_vp1_p(f-0)).
hyperedge_labels(ax_n_n(t-0)).
hyperedge_labels(ax_n_p(f-0)).
hyperedge_labels(ax_s_n(t-0)).
hyperedge_labels(ax_s_p(f-0)).

% ====================
% =       rules      =
% ====================

% = start

rule(s, [t(X,Y)], [h(X), c(Y)]).

% = tensor rules =
%
% new hypothesis on left

rule(t(X,Y), [v(X5,Y),t(X,X4),t(X3,X2)], [circ(X2,X4,X5),h(X3)]).
rule(t(X,Y), [t(X,X4),t(X3,X2)], [circ(X2,X4,Y),h(X3)]).
rule(t(X,Y), [v(X5,Y),t(X3,X2)], [circ(X2,X,X5),h(X3)]).
rule(t(X,Y), [v(X5,Y),t(X,X4)], [circ(X3,X4,X5),h(X3)]).
rule(t(X,Y), [t(X3,X2)], [circ(X2,X,Y),h(X3)]).
rule(t(X,Y), [v(X5,Y)], [circ(X3,X,X5),h(X3)]).
rule(t(X,Y), [t(X,X4)], [circ(X3,X4,Y),h(X3)]).
rule(t(X,Y), [], [circ(X3,X,Y),h(X3)]).

% new hypothesis on right

rule(t(X,Y), [v(X5,Y),t(X,X2),t(X3,X4)], [circ(X2,X4,X5),h(X3)]).
rule(t(X,Y), [t(X,X2),t(X3,X4)], [circ(X2,X4,Y),h(X3)]).
rule(t(X,Y), [v(X5,Y),t(X3,X4)], [circ(X,X4,X5),h(X3)]).
rule(t(X,Y), [v(X5,Y),t(X,X2)], [circ(X2,X3,X5),h(X3)]).
rule(t(X,Y), [t(X3,X4)], [circ(X,X4,Y),h(X3)]).
rule(t(X,Y), [v(X5,Y)], [circ(X,X3,X5),h(X3)]).
rule(t(X,Y), [t(X,X2)], [circ(X2,X3,Y),h(X3)]).
rule(t(X,Y), [], [circ(X,X3,Y),h(X3)]).

% = tree to vertex swap

rule(t(X,Y), [v(X,Y)], []).

% = par rules =
%
% mode 0

% = left rule for product - lprod

rule(v(X,Y), [v(X4,Y),v(X2,X0),v(X3,X1)], [lprod(X2,X3,X),circ(X0,X1,X4)]).
rule(v(X,Y), [v(X2,X0),v(X3,X1)], [lprod(X2,X3,X),circ(X0,X1,Y)]).
rule(v(X,Y), [v(X4,Y),v(X3,X1)], [lprod(X0,X3,X),circ(X0,X1,X4)]).
rule(v(X,Y), [v(X4,Y),v(X2,X0)], [lprod(X2,X1,X),circ(X0,X1,X4)]).
rule(v(X,Y), [v(X3,X1)], [lprod(X0,X3,X),circ(X0,X1,Y)]).
rule(v(X,Y), [v(X4,Y)], [lprod(X0,X1,X),circ(X0,X1,X4)]).
rule(v(X,Y), [v(X2,X0)], [lprod(X2,X1,X),circ(X0,X1,Y)]).
rule(v(X,Y), [], [lprod(X0,X1,X),circ(X0,X1,Y)]).

% = right rule for division left - rdl

rule(v(X,Y), [v(X,X1),v(X4,X0),v(X2,X3)], [rdl(X4,Y,X3),circ(X0,X1,X2)]).
rule(v(X,Y), [v(X4,X0),v(X2,X3)], [rdl(X4,Y,X3),circ(X0,X,X2)]).
rule(v(X,Y), [v(X,X1),v(X2,X3)], [rdl(X0,Y,X3),circ(X0,X1,X2)]).
rule(v(X,Y), [v(X,X1),v(X4,X0)], [rdl(X4,Y,X2),circ(X0,X1,X2)]).
rule(v(X,Y), [v(X2,X3)], [rdl(X0,Y,X3),circ(X0,X,X2)]).
rule(v(X,Y), [v(X,X1)], [rdl(X0,Y,X2),circ(X0,X1,X2)]).
rule(v(X,Y), [v(X4,X0)], [rdl(X4,Y,X2),circ(X0,X,X2)]).
rule(v(X,Y), [], [rdl(X0,Y,X2),circ(X0,X,X2)]).

% = right rule for division right - rdr

rule(v(X,Y), [v(X,X0),v(X4,X1),v(X2,X3)], [rdr(Y,X4,X3),circ(X0,X1,X2)]).
rule(v(X,Y), [v(X4,X1),v(X2,X3)], [rdr(Y,X4,X3),circ(X,X1,X2)]).
rule(v(X,Y), [v(X,X0),v(X2,X3)], [rdr(Y,X1,X3),circ(X0,X1,X2)]).
rule(v(X,Y), [v(X,X0),v(X4,X1)], [rdr(Y,X4,X2),circ(X0,X1,X2)]).
rule(v(X,Y), [v(X2,X3)], [rdr(Y,X1,X3),circ(X,X1,X2)]).
rule(v(X,Y), [v(X,X0)], [rdr(Y,X1,X2),circ(X0,X1,X2)]).
rule(v(X,Y), [v(X4,X1)], [rdr(Y,X4,X2),circ(X,X1,X2)]).
rule(v(X,Y), [], [rdr(Y,X1,X2),circ(X,X1,X2)]).

% = right rule for Grishin product - rplus

rule(v(X,Y), [v(X,X0),v(X2,X4),v(X1,X3)], [rplus(X3,X4,Y),semi(X1,X2,X0)]).
rule(v(X,Y), [v(X2,X4),v(X1,X3)], [rplus(X3,X4,Y),semi(X1,X2,X)]).
rule(v(X,Y), [v(X,X0),v(X1,X3)], [rplus(X3,X2,Y),semi(X1,X2,X0)]).
rule(v(X,Y), [v(X,X0),v(X2,X4)], [rplus(X1,X4,Y),semi(X1,X2,X0)]).
rule(v(X,Y), [v(X1,X3)], [rplus(X3,X2,Y),semi(X1,X2,X)]).
rule(v(X,Y), [v(X,X0)], [rplus(X1,X2,Y),semi(X1,X2,X0)]).
rule(v(X,Y), [v(X2,X4)], [rplus(X1,X4,Y),semi(X1,X2,X)]).
rule(v(X,Y), [], [rplus(X1,X2,Y),semi(X1,X2,X)]).

% = left rule for Grishin division left - lgl

rule(t(X,Y), [t(X1,X2),v(X3,X0),v(X4,Y)], [lgl(X0,X,X1),semi(X3,X4,X2)]).
rule(t(X,Y), [v(X3,X0),v(X4,Y)], [lgl(X0,X,X1),semi(X3,X4,X1)]).
rule(t(X,Y), [t(X1,X2),v(X4,Y)], [lgl(X0,X,X1),semi(X0,X4,X2)]).
rule(t(X,Y), [t(X1,X2),v(X3,X0)], [lgl(X0,X,X1),semi(X3,Y,X2)]).
rule(t(X,Y), [v(X4,Y)], [lgl(X0,X,X1),semi(X0,X4,X1)]).
rule(t(X,Y), [t(X1,X2)], [lgl(X0,X,X1),semi(X0,Y,X2)]).
rule(t(X,Y), [v(X3,X0)], [lgl(X0,X,X1),semi(X3,Y,X1)]).
rule(t(X,Y), [], [lgl(X0,X,X1),semi(X0,Y,X1)]).

% = left rule for Grishin division right - lgr

rule(t(X,Y), [t(X1,X2),v(X3,Y),v(X4,X0)], [lgr(X,X0,X1),semi(X3,X4,X2)]).
rule(t(X,Y), [v(X3,Y),v(X4,X0)], [lgr(X,X0,X1),semi(X3,X4,X1)]).
rule(t(X,Y), [t(X1,X2),v(X4,X0)], [lgr(X,X0,X1),semi(Y,X4,X2)]).
rule(t(X,Y), [t(X1,X2),v(X3,Y)], [lgr(X,X0,X1),semi(X3,X0,X2)]).
rule(t(X,Y), [v(X4,X0)], [lgr(X,X0,X1),semi(Y,X4,X1)]).
rule(t(X,Y), [t(X1,X2)], [lgr(X,X0,X1),semi(Y,X0,X2)]).
rule(t(X,Y), [v(X3,Y)], [lgr(X,X0,X1),semi(X3,X0,X1)]).
rule(t(X,Y), [], [lgr(X,X0,X1),semi(Y,X0,X1)]).

% = axiom rules for s, np, n, pp, vp, vp1

rule(v(X,Y), [], [ax_s_n(X),ax_s_p(Y)]).
rule(v(X,Y), [], [ax_np_n(X),ax_np_p(Y)]).
rule(v(X,Y), [], [ax_pp_n(X),ax_pp_p(Y)]).
rule(v(X,Y), [], [ax_vp_n(X),ax_vp_p(Y)]).
rule(v(X,Y), [], [ax_vp1_n(X),ax_vp1_p(Y)]).
rule(v(X,Y), [], [ax_n_n(X),ax_n_p(Y)]).

% ====================
% =  example graphs  =
% ====================

% identities

example(1, G) :-
	lg_sequent_to_hypergraph([g(np,n)], g(np,n), G, []).
example(2, G) :-
	lg_sequent_to_hypergraph([gl(np,n)], gl(np,n), G, []).
example(3, G) :-
	lg_sequent_to_hypergraph([gr(np,n)], gr(np,n), G, []).

% unit, co-unit, lifting

example(4, G) :-
	lg_sequent_to_hypergraph([np], g(gr(np,n),n), G, []).
example(5, G) :-
	lg_sequent_to_hypergraph([np], g(n,gl(n,np)), G, []).
example(6, G) :-
	lg_sequent_to_hypergraph([gl(n,g(n,np))], np, G, []).
example(7, G) :-
	lg_sequent_to_hypergraph([gr(g(np,n),n)], np, G, []).
example(8, G) :-
	lg_sequent_to_hypergraph([gr(s,gl(np,s))], np, G, []).
example(9, G) :-
	lg_sequent_to_hypergraph([gl(gr(s,np),s)], np, G, []).

% more linguistic examples

example(10, G) :-
	lg_sequent_to_hypergraph([np, dl(np,s)], s, G, []).
example(11, G) :-
	lg_sequent_to_hypergraph([np, dr(dl(np,s),np), np], s, G, []).
example(12, G) :-
	lg_sequent_to_hypergraph([dr(s,dl(np,s)),dl(np,s)], s, G, []).
example(13, G) :-
	lg_sequent_to_hypergraph([dr(s,dl(np,s)),dl(dr(s,dl(np,s)),s)], s, G, []).

% quantifier scope

example(14, G) :-
	lg_sequent_to_hypergraph([np, dr(dl(np,s),s), gl(gr(s,s),np), dl(np,s)], s, G, []).

% Dutch verb clusters

example(15, G) :-
	lg_sequent_to_hypergraph([np,np,dr(np,n),n,
	                          gl(gr(vp,dl(np,dl(np,s))),vp1),
				  dl(vp1,dl(np,vp))], s, G, []).

example(16, G) :-
	lg_sequent_to_hypergraph([np,np,np,dr(np,n),n,
	                          gl(gr(vp,dl(np,dl(np,s))),vp1),
				  dl(vp1,gl(gr(vp,dl(np,vp)),vp1)),
				  dl(vp1,dl(np,vp))], s, G, []).
