
% ====================
% =       NL<>R      =
% ====================

% Hyperedge replacement grammar for the non-associative Lambek calculus
% with the mixed associativity and commutativity structural rules as
% described in my paper "Lambek Grammars, Hyperedge Replacement Grammars
% and Tree Adjoining Grammars".
%
% Copyright (C) 2008 Richard Moot (Richard.Moot@labri.fr)
% Copyright (C) 2008 CNRS         (http://www.cnrs.fr)
% Copyright (C) 2008 INRIA        (http://www.inria.fr

% This library is free software; you can redistribute it and/or
% modify it under the terms of the GNU Lesser General Public
% License as published by the Free Software Foundation; either
% version 2.1 of the License, or (at your option) any later version.

% This library is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% Lesser General Public License for more details.

% You should have received a copy of the GNU Lesser General Public
% License along with this library; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

% ====================
% =   start symbol   =
% ====================

start(s).

% ====================
% =   nonterminals   =
% ====================

nonterminal(s).
nonterminal(t(_,_)).
nonterminal(v(_,_)).

% ====================
% =     terminals    =
% ====================

terminal(h(_)).
terminal(c(_)).

terminal(ax_np_n(_)).
terminal(ax_np_p(_)).
terminal(ax_vp_n(_)).
terminal(ax_vp_p(_)).
terminal(ax_vp1_n(_)).
terminal(ax_vp1_p(_)).
terminal(ax_n_n(_)).
terminal(ax_n_p(_)).
terminal(ax_s_n(_)).
terminal(ax_s_p(_)).

terminal(circ0(_,_,_)).
terminal(lprod0(_,_,_)).
terminal(rdr0(_,_,_)).
terminal(rdl0(_,_,_)).

terminal(circ1(_,_,_)).
terminal(lprod1(_,_,_)).
terminal(rdr1(_,_,_)).
terminal(rdl1(_,_,_)).

% ====================
% = hyperedge labels =
% ====================

hyperedge_labels(s).

hyperedge_labels(t(t-1,f-0)).
hyperedge_labels(v(t-1,f-0)).

hyperedge_labels(h(f-0)).
hyperedge_labels(c(t-0)).

hyperedge_labels(circ0(t-1,t-2,f-0)).
hyperedge_labels(lprod0(f-1,f-2,t-0)).
hyperedge_labels(rdr0(f-1,f-2,t-0)).
hyperedge_labels(rdl0(f-1,f-2,t-0)).

hyperedge_labels(circ1(t-1,t-2,f-0)).
hyperedge_labels(lprod1(f-1,f-2,t-0)).
hyperedge_labels(rdr1(f-1,f-2,t-0)).
hyperedge_labels(rdl1(f-1,f-2,t-0)).

hyperedge_labels(ax_np_n(t-0)).
hyperedge_labels(ax_np_p(f-0)).
hyperedge_labels(ax_pp_n(t-0)).
hyperedge_labels(ax_pp_p(f-0)).
hyperedge_labels(ax_vp_n(t-0)).
hyperedge_labels(ax_vp_p(f-0)).
hyperedge_labels(ax_vp1_n(t-0)).
hyperedge_labels(ax_vp1_p(f-0)).
hyperedge_labels(ax_n_n(t-0)).
hyperedge_labels(ax_n_p(f-0)).
hyperedge_labels(ax_s_n(t-0)).
hyperedge_labels(ax_s_p(f-0)).

% ====================
% =       rules      =
% ====================

% = start

rule(s, [t(X,Y)], [h(X), c(Y)]).

% = tensor rules =
%
% new hypothesis on left

rule(t(X,Y), [t(X5,Y),t(X,X4),t(X3,X2)], [circ0(X2,X4,X5),h(X3)]).
rule(t(X,Y), [t(X,X4),t(X3,X2)], [circ0(X2,X4,Y),h(X3)]).
rule(t(X,Y), [t(X5,Y),t(X3,X2)], [circ0(X2,X,X5),h(X3)]).
rule(t(X,Y), [t(X5,Y),t(X,X4)], [circ0(X3,X4,X5),h(X3)]).
rule(t(X,Y), [t(X3,X2)], [circ0(X2,X,Y),h(X3)]).
rule(t(X,Y), [t(X5,Y)], [circ0(X3,X,X5),h(X3)]).
rule(t(X,Y), [t(X,X4)], [circ0(X3,X4,Y),h(X3)]).
rule(t(X,Y), [], [circ0(X3,X,Y),h(X3)]).

% new hypothesis on right

rule(t(X,Y), [t(X5,Y),t(X,X2),t(X3,X4)], [circ0(X2,X4,X5),h(X3)]).
rule(t(X,Y), [t(X,X2),t(X3,X4)], [circ0(X2,X4,Y),h(X3)]).
rule(t(X,Y), [t(X5,Y),t(X3,X4)], [circ0(X,X4,X5),h(X3)]).
rule(t(X,Y), [t(X5,Y),t(X,X2)], [circ0(X2,X3,X5),h(X3)]).
rule(t(X,Y), [t(X,X2)], [circ0(X2,X3,Y),h(X3)]).
rule(t(X,Y), [t(X3,X4)], [circ0(X,X4,Y),h(X3)]).
rule(t(X,Y), [t(X5,Y)], [circ0(X,X3,X5),h(X3)]).
rule(t(X,Y), [], [circ0(X,X3,Y),h(X3)]).

% = left rule for product - lprod1

% This is the crucial case for the embedding of tree adjoining grammars
% in NL<>R: where in the "no adjunction" case below, the hyperedge
% connecting X3 and X1 is labeled "v11", here this same edge is labeled
% "t11" indicating we can insert a graph contracting to tree here and
% not just an graph contracting to a vertex. The result is therefore
% also a tree and not a vertex.

rule(t(X,Y), [t(X3,X1),v(X4,Y),v(X2,X0)], [lprod1(X2,X3,X),circ1(X0,X1,X4)]).
rule(t(X,Y), [v(X4,Y),v(X2,X0)], [lprod1(X2,X1,X),circ1(X0,X1,X4)]).
rule(t(X,Y), [t(X3,X1),v(X2,X0)], [lprod1(X2,X3,X),circ1(X0,X1,Y)]).
rule(t(X,Y), [t(X3,X1),v(X4,Y)], [lprod1(X0,X3,X),circ1(X0,X1,X4)]).
rule(t(X,Y), [v(X2,X0)], [lprod1(X2,X1,X),circ1(X0,X1,Y)]).
rule(t(X,Y), [t(X3,X1)], [lprod1(X0,X3,X),circ1(X0,X1,Y)]).
rule(t(X,Y), [v(X4,Y)], [lprod1(X0,X1,X),circ1(X0,X1,X4)]).
rule(t(X,Y), [], [lprod1(X0,X1,X),circ1(X0,X1,Y)]).

% = tree to vertex swap

rule(t(X,Y), [v(X,Y)], []).

% = par rules =
%
% mode 0

% = left rule for product - lprod0

rule(v(X,Y), [v(X4,Y),v(X2,X0),v(X3,X1)], [lprod0(X2,X3,X),circ0(X0,X1,X4)]).
rule(v(X,Y), [v(X2,X0),v(X3,X1)], [lprod0(X2,X3,X),circ0(X0,X1,Y)]).
rule(v(X,Y), [v(X4,Y),v(X3,X1)], [lprod0(X0,X3,X),circ0(X0,X1,X4)]).
rule(v(X,Y), [v(X4,Y),v(X2,X0)], [lprod0(X2,X1,X),circ0(X0,X1,X4)]).
rule(v(X,Y), [v(X3,X1)], [lprod0(X0,X3,X),circ0(X0,X1,Y)]).
rule(v(X,Y), [v(X4,Y)], [lprod0(X0,X1,X),circ0(X0,X1,X4)]).
rule(v(X,Y), [v(X2,X0)], [lprod0(X2,X1,X),circ0(X0,X1,Y)]).
rule(v(X,Y), [], [lprod0(X0,X1,X),circ0(X0,X1,Y)]).

% = right rule for division left - rdl0

rule(v(X,Y), [v(X,X1),v(X4,X0),v(X2,X3)], [rdl0(X4,Y,X3),circ0(X0,X1,X2)]).
rule(v(X,Y), [v(X4,X0),v(X2,X3)], [rdl0(X4,Y,X3),circ0(X0,X,X2)]).
rule(v(X,Y), [v(X,X1),v(X2,X3)], [rdl0(X0,Y,X3),circ0(X0,X1,X2)]).
rule(v(X,Y), [v(X,X1),v(X4,X0)], [rdl0(X4,Y,X2),circ0(X0,X1,X2)]).
rule(v(X,Y), [v(X2,X3)], [rdl0(X0,Y,X3),circ0(X0,X,X2)]).
rule(v(X,Y), [v(X,X1)], [rdl0(X0,Y,X2),circ0(X0,X1,X2)]).
rule(v(X,Y), [v(X4,X0)], [rdl0(X4,Y,X2),circ0(X0,X,X2)]).
rule(v(X,Y), [], [rdl0(X0,Y,X2),circ0(X0,X,X2)]).

% = right rule for division right - rdr0

rule(v(X,Y), [v(X,X0),v(X4,X1),v(X2,X3)], [rdr0(Y,X4,X3),circ0(X0,X1,X2)]).
rule(v(X,Y), [v(X4,X1),v(X2,X3)], [rdr0(Y,X4,X3),circ0(X,X1,X2)]).
rule(v(X,Y), [v(X,X0),v(X2,X3)], [rdr0(Y,X1,X3),circ0(X0,X1,X2)]).
rule(v(X,Y), [v(X,X0),v(X4,X1)], [rdr0(Y,X4,X2),circ0(X0,X1,X2)]).
rule(v(X,Y), [v(X2,X3)], [rdr0(Y,X1,X3),circ0(X,X1,X2)]).
rule(v(X,Y), [v(X,X0)], [rdr0(Y,X1,X2),circ0(X0,X1,X2)]).
rule(v(X,Y), [v(X4,X1)], [rdr0(Y,X4,X2),circ0(X,X1,X2)]).
rule(v(X,Y), [], [rdr0(Y,X1,X2),circ0(X,X1,X2)]).

% = right rule for division left - rdl1

rule(v(X,Y), [v(X,X1),v(X4,X0),v(X2,X3)], [rdl0(X4,Y,X3),circ1(X0,X1,X2)]).
rule(v(X,Y), [v(X4,X0),v(X2,X3)], [rdl0(X4,Y,X3),circ1(X0,X,X2)]).
rule(v(X,Y), [v(X,X1),v(X2,X3)], [rdl0(X0,Y,X3),circ1(X0,X1,X2)]).
rule(v(X,Y), [v(X,X1),v(X4,X0)], [rdl0(X4,Y,X2),circ1(X0,X1,X2)]).
rule(v(X,Y), [v(X2,X3)], [rdl0(X0,Y,X3),circ1(X0,X,X2)]).
rule(v(X,Y), [v(X,X1)], [rdl0(X0,Y,X2),circ1(X0,X1,X2)]).
rule(v(X,Y), [v(X4,X0)], [rdl0(X4,Y,X2),circ1(X0,X,X2)]).
rule(v(X,Y), [], [rdl0(X0,Y,X2),circ1(X0,X,X2)]).

% = right rule for division right - rdr1

rule(v(X,Y), [v(X,X0),v(X4,X1),v(X2,X3)], [rdr0(Y,X4,X3),circ1(X0,X1,X2)]).
rule(v(X,Y), [v(X4,X1),v(X2,X3)], [rdr0(Y,X4,X3),circ1(X,X1,X2)]).
rule(v(X,Y), [v(X,X0),v(X2,X3)], [rdr0(Y,X1,X3),circ1(X0,X1,X2)]).
rule(v(X,Y), [v(X,X0),v(X4,X1)], [rdr0(Y,X4,X2),circ1(X0,X1,X2)]).
rule(v(X,Y), [v(X2,X3)], [rdr0(Y,X1,X3),circ1(X,X1,X2)]).
rule(v(X,Y), [v(X,X0)], [rdr0(Y,X1,X2),circ1(X0,X1,X2)]).
rule(v(X,Y), [v(X4,X1)], [rdr0(Y,X4,X2),circ1(X,X1,X2)]).
rule(v(X,Y), [], [rdr0(Y,X1,X2),circ1(X,X1,X2)]).

% = axiom rules for s, np, n, pp, vp, vp1

rule(v(X,Y), [], [ax_s_n(X),ax_s_p(Y)]).
rule(v(X,Y), [], [ax_np_n(X),ax_np_p(Y)]).
rule(v(X,Y), [], [ax_pp_n(X),ax_pp_p(Y)]).
rule(v(X,Y), [], [ax_vp_n(X),ax_vp_p(Y)]).
rule(v(X,Y), [], [ax_vp1_n(X),ax_vp1_p(Y)]).
rule(v(X,Y), [], [ax_n_n(X),ax_n_p(Y)]).

% ====================
% =  example graphs  =
% ====================

% = identities

example(1, G) :-
	sequent_to_hypergraph([p(0,np,n)], p(0,np,n), G, []).
example(2, G) :-
	sequent_to_hypergraph([dl(0,np,n)], dl(0,np,n), G, []).
example(3, G) :-
	sequent_to_hypergraph([dr(0,np,n)], dr(0,np,n), G, []).

% = unit, co-unit, lifting

example(4, G) :-
	sequent_to_hypergraph([p(0,dr(0,np,n),n)], np, G, []).
example(5, G) :-
	sequent_to_hypergraph([p(0,n,dl(0,n,np))], np, G, []).
example(6, G) :-
	sequent_to_hypergraph([np], dl(0,n,p(0,n,np)), G, []).
example(7, G) :-
	sequent_to_hypergraph([np], dr(0,p(0,np,n),n), G, []).
example(8, G) :-
	sequent_to_hypergraph([np], dr(0,s,dl(0,np,s)), G, []).
example(9, G) :-
	sequent_to_hypergraph([np], dl(0,dr(0,s,np),s), G, []).

% "John walks."
example(10, G) :-
	sequent_to_hypergraph([np, dl(0,np,s)], s, G, []).
% "John loves Mary."
example(11, G) :-
	sequent_to_hypergraph([np, dr(0,dl(0,np,s),np), np], s, G, []).
% "Someone is_missing."
example(12, G) :-
	sequent_to_hypergraph([dr(0,s,dl(0,np,s)),dl(0,dr(0,s,dl(0,np,s)),s)], s, G, []).

% = quantifier scope

% "Someone walks."
example(13, G) :-
	sequent_to_hypergraph([p(1,dr(1,s,s),np),dl(0,np,s)], s, G, []).
% "Someone likes Mary."
example(14, G) :-
	sequent_to_hypergraph([p(1,dr(1,s,s),np),dr(0,dl(0,np,s),np),np], s, G, []).
% "John likes someone."
example(15, G) :-
	sequent_to_hypergraph([np,dr(0,dl(0,np,s),np),p(1,dr(1,s,s),np)], s, G, []).
% "John gave someone money."
example(16, G) :-
	sequent_to_hypergraph([np,dr(0,dr(0,dl(0,np,s),np),np),p(1,dr(1,s,s),np),np], s, G, []).
% "John believes someone left."
example(17, G) :-
	sequent_to_hypergraph([np, dr(0,dl(0,np,s),s), p(1,dr(1,s,s),np), dl(0,np,s)], s, G, []).
example(18, G) :-
% "Someone loves everyone."
	sequent_to_hypergraph([p(1,dr(1,s,s),np), dr(0,dl(0,np,s),np), p(1,dr(1,s,s),np)], s, G, []).

% Dutch verb clusters

% "(dat) ik Cecilia de nijlpaarden zag voeren."
example(19, G) :-
	sequent_to_hypergraph([np,np,np,p(1,dr(1,dl(0,np,dl(0,np,s)),vp),vp1), dl(0,vp1,dl(0,np,vp))], s, G, []).
% "(dat) ik Cecilia Henk de nijlpaarden zag helpen voeren."
example(20, G) :-
	sequent_to_hypergraph([np,np,np,np,p(1,dr(1,dl(0,np,dl(0,np,s)),vp),vp1), dl(0,vp1,p(1,dr(1,dl(0,np,vp),vp),vp1)), dl(0,vp1,dl(0,np,vp))], s, G, []).